# 一、Nodejs 基础

## 安装nodejs 

soft.huruqing.cn

## (一) 第一个 node 应用

1. #### 新建 app.js,代码如下

```JS
var http = require('http');
http.createServer(function (request, response) {

    // 发送 HTTP 头部
    // HTTP 状态值: 200 : OK
    // 内容类型: text/plain
    response.writeHead(200, {
        'Content-Type': 'text/plain'
    });

    response.end('Hello World');
}).listen(8888);

// 终端打印如下信息
console.log('Server running at http://127.0.0.1:8888/');
```

2. #### 启动服务

```JS
node app.js
```



## (二) nodejs原生路由,创建多个接口

路由介绍: 路由字面意思是"途经", 路途经过什么地方的意思，没有两个地方的风景是一样, 所以你经过不同的地方, 看到的风景是不一样的,网站上的路由也类似这样的意思, 你访问不同的url, 那么你看到的东西也是不一样的。

```javascript
// require获取http模块
var http = require("http"); 
// req request缩写, 存放的是请求的相关信息
http
  .createServer((req, res) => {
    // 允许跨域访问
    res.setHeader("Access-Control-Allow-Origin", "*");
    console.log('前端的请求:', req.url);

    const list = [
        {name: 'zs',age:18},
        {name: 'ls',age:19},
        {name: 'ww',age:20},
    ];

    switch (req.url) {
      case "/list":
        res.write(JSON.stringify(list));
        res.end();
        break;
      case "/hehe":
        res.write("hehe");
        res.end();
        break;
      case "/my":
        res.write("my");
        res.end();
        break;
      default:
        res.write("this is home");
        res.end();
        break;
    }
  })
  .listen(3000, "localhost", () => {
    console.log("服务器运行在：http://localhost:3000");
  });

```

```javascript
// 运行代码
node app.js
// 访问以下地址看看有什么不一样 
http://localhost:3000/list
http://localhost:3000/hehe
http://localhost:3000/my
http://localhost:3000
```



## (三) nodejs 的模块系统

**1. nodejs模块系统介绍**

一个js文件就是一个模块, 一个模块会有一些变量或者方法,可以导出这些变量或者方法提供给别的模块进行使用, 它也可以导入别的模块或者方法来使用 

**2. 导出模块和导入模块**

* 导出用`module.epxorts`
* 导入用`require()`

```javascript
// utils.js 导出
module.exports = {
    // 把对象转成json字符串
    toJson(data) {
        return JSON.stringify(data);
    }
}
```

```JavaScript
// demo.js, 导入utils.js, 并调用它的toJson方法
const utils = require('./utils');
var obj = {
    username: '张三',
    age:18
}

var str = utils.toJson(obj);
console.log(typeof str);

```



## (四) 文件系统

1. #### 读取文件

   demo.txt文件
   
   ```
   I love web
   ```
   
   demo.js文件
   
   ```js
   // fs是nodejs提供用来读写文件的模块
   const fs = require('fs');
   // readFileSync同步读取文件的方法
   let file = fs.readFileSync('./demo.txt');
   console.log(file);
   console.log(file.toString());
   ```
   
2. #### 写入文件

   ```js
   const fs = require('fs');
   fs.writeFileSync('demo2.txt','我爱web前端');
   
   let list = [];
   for(let i=0;i<100;i++) {
       list.push({
           name: '张三'+i,
           age: i
       })
   }
   fs.writeFileSync('user.json', JSON.stringify(list))
   ```




## (五) path模块

path模块介绍: nodejs自带的模块, 主要用来解决路径相关问题

在同一个文件夹内, 打开cdm终端和bash终端, 输入`pwd`看看结果有什么区别

* 在git bash 执行 pwd命令(查看所在位置)

  ```
  /c/Users/Administrator/Desktop/code.huruqing.cn/proj/web
  ```

* 在cmd  执行pwd

  ```
  C:\Users\Administrator\Desktop\code.huruqing.cn\proj\web>
  ```

  

1. path.join(), 因为window和linux系统分割符不一致,join可以解决这个问题

```js
const path = require('path');
let str1 = path.join('/c/Users/Administrator/Desktop/code.huruqing.cn/proj/server', 'dist');
console.log(str1);
```

2. __dirname 当前目录

```js
const path = require('path');
let str1 = path.join('/c/Users/Administrator/Desktop/code.huruqing.cn/proj/server', 'dist');
let str2 = path.join(__dirname,'dist');
console.log(str1);
console.log(str2);
```

## (六) npm 工具

1. #### npm 是什么

   * 前端通用的模块都存放在网站  https://www.npmjs.com/, 你需要用到nodejs模块在这个网站都能找到
   * npm 是 nodejs 模块 (一般称为包,一个模块就是一个包) 管理工具

2. #### 初始化 package.json

   ```typescript
   npm init   // 或
   npm init -y
   ```

   版本说明

    "boo": "2.3.1", 安装2.3.1的版本

   "boo": "~2.3.1" 安装>=2.3.1 < 2.4里的最高的版本

    "boo": "^2.3.1",   安装的版本>=2.3.1 < 3.0.0, 会安装最高的版本

   ```js
   // package.json 版本号说明
   {
       "dependencies": {
           "foo": "1.0.0 - 2.9999.9999",   
           "bar": ">=1.0.2 <2.1.2", //必须大于等于1.0.2版本且小于2.1.2版本
           "baz": ">1.0.2 <=2.3.4", //必须大于1.0.2版本且小于等于2.3.4版本
           "boo": "2.3.1", //必须匹配这个版本
           "boo": "~2.3.1", //约等于2.3.1，只更新最小版本，相当于2.3.X，即>=2.3.1 <2.4.0
           "thr": "2.3.x",
           "boo": "^2.3.1", //与2.3.1版本兼容，相当于2.X.X, 即>=2.3.1 < 3.0.0,不改变大版本号。
           "qux": "<1.0.0 || >=2.3.1 <2.4.5 || >=2.5.2 <3.0.0",
            //在版本上指定一个压缩包的url，当执行npm install 时这个压缩包会被下载并安装到本地。
           "asd": "http://asdf.com/asdf.tar.gz", 
           "til": "~1.2",   
           "elf": "~1.2.3", 
           "two": "2.x",
           "lat": "latest",              //安装最新版本
           "dyl": "file:../dyl",         //使用本地路径
            // 使用git URL加commit-ish
           "adf": "git://github.com/user/project.git#commit-ish"    
       }
   } 
   ```

   

3. #### 安装一个 nodejs 模块

   * npm install (简写 npm i) 安装模块, 安装好的模块就放在node_modules文件夹里
   * npm i xxx 和 npm i --save 记录在package.json的dependencies里
   * npm i xxx --save-dev  记录在package.json的devDependencies里
   * npm i xxx@xx.xx.xx  安装指定版本
   * npm i   可以重新安装package.json里记录的模块

   ```typescript
   // 以jquery和axios为例    
   npm i jquery 和 npm i jquery --save  // 记录在 dependencies
   npm i axios --save-dev  //  记录devDependencies
   npm i serve -g    // 全局安装,serve是一个封装了nodejs服务的模块
   npm i // 当我们把node_modules删掉,使用npm i 可以把所有依赖都安装回来
   ```

   dependency和devDependencies的区别

   dependency生产依赖, devDependencies开发依赖

4. #### 更新 nodejs 模块

   ```
   npm update xxx  // 简写 npm i xx
   ```

5. #### 删除 nodejs 模块

   ```
   npm uninstall xxx // 简写 npm un xx
   ```

6. #### 运行指令 (运行package.json里script里的指令)

   比如package.json里有以下代码

   ```JavaScript
    "scripts": {
        "dev": "nodemon app.js"
     },
   ```

   那么就可以使用 `npm run dev`来执行 `nodemon app.js`

## (七) 设置淘宝镜像

(1) 输入以下命令

```
npm config set registry https://registry.npm.taobao.org  
```

(2) 验证

```
npm config get registry
```

(3) 如果返回https://registry.npm.taobao.org，说明镜像配置成功。

## (八) nodejs版本切换   
1. #### 下载安装 nvm
```
https://nvm.en.softonic.com/
```
2. #### nvm命令

```typescript
nvm                  // 会提示nvw下的相关命令
nvm ls               // 查看已安装node版本
nvm install 10      // 安装对应10.0版本的node
nvm uninstall 10    // 卸载对应10.0版本的node
nvm use xxx          // 选择使用XXX版本
```

## (九) 服务器自动重启

1. 全局安装`nodemon`  `npm i nodemon -g`
2. 运行`node app.js`需要使用`nodemon app.js`来代替
3. 示例: 新建`app.js`, 然后执行 `nodemon app.js`, 然后修改输出内容试试

```javascript
var http = require('http');
http.createServer(function (request, response) {

    // 发送 HTTP 头部
    // HTTP 状态值: 200 : OK
    // 内容类型: text/plain
    response.writeHead(200, {
        'Content-Type': 'text/plain'
    });

    response.end('Hello World');
}).listen(8888);

// 终端打印如下信息
console.log('Server running at http://127.0.0.1:8888/');
```

## (十) 调试代码

可以使用vscode里的调试工具


